@@ -107,8 +107,14 @@ Comploader::Comploader()
   }
 }
 
-Comploader::librarymap_type Comploader::librarymap;
-const Tntconfig* Comploader::config = 0;
+Comploader::librarymap_type& Comploader::getLibrarymap()
+{
+  static librarymap_type librarymap;
+  return librarymap;
+}
+
+static const Tntconfig emptyconfig;
+const Tntconfig* Comploader::config = &emptyconfig;
 Comploader::search_path_type Comploader::search_path;
 ComponentLibrary::factoryMapType* Comploader::currentFactoryMap = 0;
 
@@ -171,12 +177,13 @@ namespace
       ~ValueResetter()
       { value = null; }
   };
-};
+}
 
 ComponentLibrary& Comploader::fetchLib(const std::string& libname)
 {
   log_debug("fetchLib \"" << libname << '"');
 
+  librarymap_type& librarymap = getLibrarymap();
   librarymap_type::iterator it = librarymap.find(libname);
   if (it == librarymap.end())
   {
@@ -204,6 +211,21 @@ ComponentLibrary& Comploader::fetchLib(const std::string& libname)
       }
     }
 
+#ifdef PKGLIBDIR
+    if (!found)
+    {
+      try
+      {
+        log_debug("load library \"" << libname << "\" from package lib dir");
+        lib = ComponentLibrary(PKGLIBDIR, libname);
+        found = true;
+      }
+      catch (const cxxtools::dl::DlopenError& e)
+      {
+      }
+    }
+#endif
+
     if (!found)
     {
       try
@@ -251,6 +273,7 @@ void Comploader::registerFactory(const std::string& component_name,
     currentFactoryMap->insert(ComponentLibrary::factoryMapType::value_type(component_name, factory));
   else
   {
+    librarymap_type& librarymap = getLibrarymap();
     log_debug("register component without library-name");
     librarymap_type::iterator it = librarymap.find(std::string());
     if (it == librarymap.end())
