@@ -41,7 +41,7 @@ namespace tnt
   size_t HttpRequest::maxRequestSize = 0;
   unsigned HttpRequest::serial_ = 0;
 
-  HttpRequest::HttpRequest()
+  HttpRequest::HttpRequest(Tntnet& application_)
     : ssl(false),
       locale_init(false),
       encodingRead(false),
@@ -50,13 +50,14 @@ namespace tnt
       threadScope(0),
       sessionScope(0),
       applicationScopeLocked(false),
-      sessionScopeLocked(false)
+      sessionScopeLocked(false),
+      application(application_)
   {
     memset(&peerAddr, 0, sizeof(peerAddr));
     memset(&serverAddr, 0, sizeof(peerAddr));
   }
 
-  HttpRequest::HttpRequest(const std::string& url)
+  HttpRequest::HttpRequest(Tntnet& application_, const std::string& url)
     : ssl(false),
       locale_init(false),
       requestScope(0),
@@ -64,7 +65,8 @@ namespace tnt
       threadScope(0),
       sessionScope(0),
       applicationScopeLocked(false),
-      sessionScopeLocked(false)
+      sessionScopeLocked(false),
+      application(application_)
   {
     std::istringstream s("GET " + url + " HTTP/1.1\r\n\r\n");
     parse(s);
@@ -87,7 +89,8 @@ namespace tnt
       threadScope(r.threadScope),
       sessionScope(r.sessionScope),
       applicationScopeLocked(false),
-      sessionScopeLocked(false)
+      sessionScopeLocked(false),
+      application(r.application)
   {
     if (requestScope)
       requestScope->addRef();
@@ -178,6 +181,16 @@ namespace tnt
     }
   }
 
+  std::string HttpRequest::getArg(const std::string& name, const std::string& def) const
+  {
+    for (args_type::const_iterator it = args.begin(); it != args.end(); ++it)
+      if (it->size() > name.size()
+        && it->compare(0, name.size(), name) == 0
+        && it->at(name.size()) == '=')
+        return it->substr(name.size() + 1);
+    return def;
+  }
+
   void HttpRequest::parse(std::istream& in)
   {
     Parser p(*this);
@@ -424,6 +437,13 @@ namespace tnt
                                               CompareNoCase());
   }
 
+  const Contenttype& HttpRequest::getContentTypePriv() const
+  {
+    std::istringstream in(getHeader(httpheader::contentType));
+    in >> ct;
+    return ct;
+  }
+
   void HttpRequest::setApplicationScope(Scope* s)
   {
     if (applicationScope == s)
