@@ -29,6 +29,14 @@ log_define("tntnet.parser")
 
 namespace tnt
 {
+  namespace
+  {
+    inline bool isVariableNameChar(char ch)
+    {
+      return std::isalnum(ch) || ch == '_';
+    }
+  }
+
   namespace ecpp
   {
     static const char split_start = '{';
@@ -133,43 +141,47 @@ namespace tnt
         state_callname_string,
         state_callname,
         state_call_cpparg0,
-        state_call_cpparg,   // 50
+        state_call_cpparg1,  // 50
+        state_call_cpparg_pe,
+        state_call_cpparg_sp,
+        state_call_cpparg_e,
         state_callend,
         state_callarg0,
         state_callarg,
         state_callarge,
         state_callval_expr,
         state_callval_string,
-        state_callval_word,
+        state_callval_word,  // 60
         state_callval0,
         state_callvale,
-        state_comment,  // 60
+        state_comment,
         state_commente,
         state_compe0,
         state_compe,
         state_cond,
         state_condexpr,
         state_condexpre,
-        state_include0,
+        state_include0,  // 70
         state_include1,
         state_scopearg0,
-        state_scopearg,  // 70
+        state_scopearg,
         state_scopeargeq,
         state_scopeargval0,
         state_scopeargval,
         state_scopevale,
         state_scope0,
         state_scope,
-        state_scopeinit,
+        state_scopeinit,  // 80
+        state_scopeiniteq,
         state_scopee,
         state_scopee0,
-        state_scopecomment0,  // 80
+        state_scopecomment0,
         state_scopecomment,
         state_endcall0,
         state_endcall,
         state_endcalle,
         state_doc,
-        state_doce
+        state_doce        // 90
       };
 
       state_type state = state_nl;
@@ -179,8 +191,9 @@ namespace tnt
       std::string cond, expr;
       comp_args_type comp_args;
       std::string pass_cgi;
-      std::string defarg, defval;
+      std::string defarg, defval, paramname;
       cppargs_type cppargs;
+      paramargs_type paramargs;
       unsigned bracket_count = 0;
       std::string scopetype, scopevar, scopeinit;
       scope_container_type scope_container;
@@ -395,6 +408,12 @@ namespace tnt
                 scope = component_scope;
                 state = state_scope0;
               }
+              else if (tag == "param")
+              {
+                scope_container = param_container;
+                scope = component_scope;
+                state = state_scope0;
+              }
               else if (!inClose && tag == "close")
               {
                 handler.startClose();
@@ -434,6 +453,11 @@ namespace tnt
               scope_container = request_container;
               state = state_scopearg0;
             }
+            else if (std::isspace(ch) && tag == "param")
+            {
+              scope_container = param_container;
+              state = state_scopearg0;
+            }
             else
               tag += ch;
             break;
@@ -708,6 +732,7 @@ namespace tnt
                 || tag == "application"
                 || tag == "thread"
                 || tag == "request"
+                || tag == "param"
                 || tag == "doc")
                 ;
               else
@@ -1045,47 +1070,143 @@ namespace tnt
           case state_callname:
             if (ch == '&' || ch == '/' || ch == '>')
             {
-              log_debug("onCall(\"" << comp << "comp_args, pass_cgi, defarg)");
-              handler.onCall(comp, comp_args, pass_cgi, defarg);
+              log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
+              handler.onCall(comp, comp_args, pass_cgi, paramargs, defarg);
               comp.clear();
               comp_args.clear();
               pass_cgi.clear();
+              paramargs.clear();
               defarg.clear();
               state = ch == '>' ? state_html : state_callend;
             }
             else if (std::isspace(ch))
               state = state_callarg0;
             else if (ch == '(')
+            {
               state = state_call_cpparg0;
+              expr.clear();
+              paramargs.clear();
+            }
             else
               comp += ch;
             break;
 
           case state_call_cpparg0:
-            if (!std::isspace(ch))
+            if (ch == ')')
+              state = state_callarg;
+            else if (std::isalpha(ch) || ch == '_')
             {
-              if (ch == ')')
-                state = state_callarg;
-              else
-              {
-                defarg = ch;
-                if (ch == '(')
-                  ++bracket_count;
-                state = state_call_cpparg;
-              }
+              expr = ch;
+              state = state_call_cpparg_pe;
+            }
+            else if (!std::isspace(ch))
+            {
+              expr = ch;
+              if (ch == '(')
+                bracket_count = 1;
+              state = state_call_cpparg_e;
             }
             break;
 
-          case state_call_cpparg:
-            if (bracket_count == 0 && ch == ')')
+          case state_call_cpparg1:
+            if (ch == ')')
+              throw parse_error("')' unexpected", state, curline);
+            else if (std::isalpha(ch) || ch == '_')
+            {
+              expr = ch;
+              state = state_call_cpparg_pe;
+            }
+            else if (!std::isspace(ch))
+            {
+              expr = ch;
+              if (ch == '(')
+                bracket_count = 1;
+              state = state_call_cpparg_e;
+            }
+            break;
+
+          case state_call_cpparg_pe:
+            if (isVariableNameChar(ch))
+              expr += ch;
+            else if (std::isspace(ch))
+            {
+              paramname = expr;
+              state = state_call_cpparg_sp;
+            }
+            else if (ch == '=')
+            {
+              paramname = expr;
+              expr.clear();
+              bracket_count = 0;
+              state = state_call_cpparg_e;
+            }
+            else if (ch == ')')
+            {
+              if (!defarg.empty())
+                defarg += ',';
+              defarg += expr;
+              expr.clear();
               state = state_callarg0;
+            }
+            else
+            {
+              expr += ch;
+              if (ch == '(')
+                bracket_count = 1;
+              state = state_call_cpparg_e;
+            }
+            break;
+
+          case state_call_cpparg_sp:
+            if (std::isspace(ch))
+              expr += ch;
+            else if (ch == '=')
+            {
+              expr.clear();
+              bracket_count = 0;
+              state = state_call_cpparg_e;
+            }
+            else
+            {
+              expr += ch;
+              paramname.clear();
+              if (ch == '(')
+                bracket_count = 1;
+              state = state_call_cpparg_e;
+            }
+            break;
+
+          case state_call_cpparg_e:
+            if (ch == ')' && bracket_count > 0)
+            {
+              --bracket_count;
+              expr += ch;
+            }
+            else if (ch == ',' || ch == ')')
+            {
+              if (paramname.empty())
+              {
+                if (!defarg.empty())
+                  defarg += ',';
+                defarg += expr;
+                expr.clear();
+                state = ch == ',' ? state_call_cpparg1 : state_callarg0;
+              }
+              else
+              {
+                if (paramargs.find(paramname) != paramargs.end())
+                  throw parse_error("duplicate parameter " + paramname, state, curline);
+                paramargs[paramname] = expr;
+                paramname.clear();
+                expr.clear();
+                state = ch == ',' ? state_call_cpparg1 : state_callarg0;
+              }
+            }
             else
             {
-              defarg += ch;
               if (ch == '(')
                 ++bracket_count;
-              else if (ch == ')')
-                --bracket_count;
+              expr += ch;
             }
             break;
 
@@ -1104,12 +1225,13 @@ namespace tnt
             }
             else if (ch == '&' || ch == '/' || ch == '>')
             {
-              log_debug("onCall(\"" << comp << "comp_args, pass_cgi, defarg)");
-              handler.onCall(comp, comp_args, pass_cgi, defarg);
+              log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
+              handler.onCall(comp, comp_args, pass_cgi, paramargs, defarg);
               arg.clear();
               comp.clear();
               comp_args.clear();
               pass_cgi.clear();
+              paramargs.clear();
               defarg.clear();
               state = ch == '>' ? state_html : state_callend;
             }
@@ -1120,7 +1242,7 @@ namespace tnt
             break;
 
           case state_callarg:
-            if (std::isalnum(ch) || ch == '_')
+            if (isVariableNameChar(ch))
               arg += ch;
             else if (std::isspace(ch))
               state = state_callarge;
@@ -1128,12 +1250,12 @@ namespace tnt
               state = state_callval0;
             else if (pass_cgi.empty() && ch == '&' || ch == '/' || ch == '>')
             {
-              log_debug("onCall(\"" << comp << "comp_args, pass_cgi, defarg)");
-              handler.onCall(comp, comp_args, arg, defarg);
+              log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
+              handler.onCall(comp, comp_args, arg, paramargs, defarg);
               arg.clear();
               comp.clear();
               comp_args.clear();
-              pass_cgi.clear();
+              paramargs.clear();
               defarg.clear();
               state = ch == '>' ? state_html : state_callend;
             }
@@ -1144,7 +1266,7 @@ namespace tnt
           case state_callarge:
             if (ch == '=')
               state = state_callval0;
-            else if (pass_cgi.empty() && (std::isalnum(ch) || ch == '_'))
+            else if (pass_cgi.empty() && (isVariableNameChar(ch)))
             {
               pass_cgi = arg;
               arg = ch;
@@ -1152,12 +1274,12 @@ namespace tnt
             }
             else if (pass_cgi.empty() && ch == '&' || ch == '/' || ch == '>')
             {
-              log_debug("onCall(\"" << comp << "comp_args, pass_cgi, defarg)");
-              handler.onCall(comp, comp_args, arg, defarg);
+              log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
+              handler.onCall(comp, comp_args, arg, paramargs, defarg);
               arg.clear();
               comp.clear();
               comp_args.clear();
-              pass_cgi.clear();
+              paramargs.clear();
               defarg.clear();
               state = ch == '>' ? state_html : state_callend;
             }
@@ -1227,11 +1349,12 @@ namespace tnt
               arg.clear();
               value.clear();
 
-              log_debug("onCall(\"" << comp << "comp_args, pass_cgi, defarg)");
-              handler.onCall(comp, comp_args, pass_cgi, defarg);
+              log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
+              handler.onCall(comp, comp_args, pass_cgi, paramargs, defarg);
               comp.clear();
               comp_args.clear();
               pass_cgi.clear();
+              paramargs.clear();
               defarg.clear();
               state = state_html;
             }
@@ -1456,6 +1579,7 @@ namespace tnt
               state = state_scopecomment0;
             else if (!std::isspace(ch))
             {
+              handler.onLine(curline, curfile); //#
               scopevar = ch;
               state = state_scope;
             }
@@ -1483,9 +1607,14 @@ namespace tnt
               state = state_scopeinit;
               bracket_count = 0;
             }
+            else if (ch == '=')
+            {
+              state = state_scopeiniteq;
+              bracket_count = 0;
+            }
             else if (std::isspace(ch))
               scopevar += ch;
-            else if (!std::isalnum(scopevar.at(scopevar.size() - 1)))
+            else if (!isVariableNameChar(scopevar.at(scopevar.size() - 1)))
             {
               scopetype += scopevar;
               scopevar = ch;
@@ -1513,9 +1642,12 @@ namespace tnt
             {
               // scopevar contains variable-definition
               // scopeinit contains constructorparameter
-              if (scopetype.size() > 0
+              while (scopetype.size() > 0
                 && std::isspace(scopetype.at(scopetype.size() - 1)))
                   scopetype.erase(scopetype.size() - 1);
+              while (scopevar.size() > 0
+                && std::isspace(scopevar.at(scopevar.size() - 1)))
+                  scopevar.erase(scopevar.size() - 1);
               log_debug("onScope(" << scope_container << ", " << scope << ", "
                   << scopetype << ", " << scopevar << ", " << scopeinit << ')');
               handler.onScope(scope_container, scope, scopetype, scopevar, scopeinit);
@@ -1531,7 +1663,45 @@ namespace tnt
               if (ch == '(')
                 ++bracket_count;
               else if (ch == ')')
+              {
+                if (bracket_count == 0)
+                  throw parse_error("unexpected ')'", state, curline);
                 --bracket_count;
+              }
+            }
+            break;
+
+          case state_scopeiniteq:
+            if (bracket_count == 0 && ch == ';')
+            {
+              // scopevar contains variable-definition
+              // scopeinit contains constructorparameter
+              while (scopetype.size() > 0
+                && std::isspace(scopetype.at(scopetype.size() - 1)))
+                  scopetype.erase(scopetype.size() - 1);
+              while (scopevar.size() > 0
+                && std::isspace(scopevar.at(scopevar.size() - 1)))
+                  scopevar.erase(scopevar.size() - 1);
+              log_debug("onScope(" << scope_container << ", " << scope << ", "
+                  << scopetype << ", " << scopevar << ", " << scopeinit << ')');
+              handler.onScope(scope_container, scope, scopetype, scopevar, scopeinit);
+
+              scopetype.clear();
+              scopevar.clear();
+              scopeinit.clear();
+              state = state_scope0;
+            }
+            else
+            {
+              scopeinit += ch;
+              if (ch == '(')
+                ++bracket_count;
+              else if (ch == ')')
+              {
+                if (bracket_count == 0)
+                  throw parse_error("unexpected ')'", state, curline);
+                --bracket_count;
+              }
             }
             break;
 
@@ -1555,7 +1725,7 @@ namespace tnt
             break;
 
           case state_endcall0:
-            if (std::isalnum(ch) || ch == '_')
+            if (isVariableNameChar(ch))
             {
               comp = ch;
               state = state_endcall;
@@ -1599,7 +1769,7 @@ namespace tnt
               comp.clear();
               state = state_endcalle;
             }
-            else if (std::isalnum(ch) || ch == '_' || ch == '@')
+            else if (isVariableNameChar(ch) || ch == '@')
               comp += ch;
             else
               throw parse_error("character expected", state, curline);
@@ -1636,6 +1806,7 @@ namespace tnt
 
         }  // switch(state)
 
+        log_debug("line " << curline << " char " << ch << " state " << state << " bc " << bracket_count);
       }  // while(in.get(ch))
 
       if (state != state_html && state != state_html0 && state != state_nl)
