@@ -166,9 +166,9 @@ namespace tnt
 
     void Generator::onCall(const std::string& comp,
       const comp_args_type& args, const std::string& pass_cgi,
-      const std::string& cppargs)
+      const paramargs_type& paramargs, const std::string& cppargs)
     {
-      currentComp->addCall(curline, curfile, comp, args, pass_cgi, cppargs);
+      currentComp->addCall(curline, curfile, comp, args, pass_cgi, paramargs, cppargs);
     }
 
     void Generator::onEndCall(const std::string& comp)
@@ -225,9 +225,10 @@ namespace tnt
     void Generator::onScope(scope_container_type container, scope_type scope,
       const std::string& type, const std::string& var, const std::string& init)
     {
+      log_debug("onScope type=\"" << type << "\" var=\"" << var << "\" init=\"" << init << '"');
       tnt::ecppc::Component* comp = (scope == ecpp::page_scope ? &maincomp : currentComp);
 
-      comp->addScopevar(Scopevar(container, scope, type, var, init));
+      comp->addScopevar(Scopevar(container, scope, type, var, init, curline, curfile));
     }
 
     void Generator::onInclude(const std::string& file)
@@ -286,22 +287,22 @@ namespace tnt
 
     void Generator::getClassDeclaration(std::ostream& out) const
     {
-      out << "class " << maincomp.getName() << " : public ";
+      out << "class _component_" << maincomp.getName() << " : public ";
       if (componentclass.empty())
         out << "tnt::EcppComponent";
       else
         out << componentclass;
       out << "\n"
              "{\n"
-             "    " << maincomp.getName() << "& main()  { return *this; }\n\n" 
+             "    _component_" << maincomp.getName() << "& main()  { return *this; }\n\n" 
              "  protected:\n"
-             "    ~" << maincomp.getName() << "();\n\n"
+             "    ~_component_" << maincomp.getName() << "();\n\n"
              "  public:\n"
-             "    " << maincomp.getName() << "(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);\n\n"
-             "    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, cxxtools::QueryParams& qparam);\n";
+             "    _component_" << maincomp.getName() << "(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);\n\n"
+             "    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);\n";
       if (haveCloseComp)
         out << "    unsigned endTag(tnt::HttpRequest& request, tnt::HttpReply& reply,\n"
-               "                    cxxtools::QueryParams& qparam);\n";
+               "                    tnt::QueryParams& qparam);\n";
       if (!attr.empty())
         out << "    std::string getAttribute(const std::string& name,\n"
                "      const std::string& def = std::string()) const;\n\n";
@@ -355,41 +356,41 @@ namespace tnt
     {
       if (configs.empty())
       {
-        code << "static tnt::ComponentFactoryImpl<" << maincomp.getName() << "> "
+        code << "static tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << "> "
              << maincomp.getName() << "Factory(\"" << maincomp.getName() << "\");\n\n";
       }
       else
       {
-        code << "class " << maincomp.getName() << "Factory : public tnt::ComponentFactoryImpl<" << maincomp.getName() << ">\n"
+        code << "class _component_" << maincomp.getName() << "Factory : public tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << ">\n"
                 "{\n"
                 "  public:\n"
-                "    " << maincomp.getName() << "Factory()\n"
-                "      : tnt::ComponentFactoryImpl<" << maincomp.getName() << ">(\"" << maincomp.getName() << "\")\n"
+                "    _component_" << maincomp.getName() << "Factory()\n"
+                "      : tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << ">(\"" << maincomp.getName() << "\")\n"
                 "      { }\n"
                 "    tnt::Component* doCreate(const tnt::Compident& ci,\n"
                 "      const tnt::Urlmapper& um, tnt::Comploader& cl);\n"
                 "    virtual void doConfigure(const tnt::Tntconfig& config);\n"
                 "};\n\n"
-                "tnt::Component* " << maincomp.getName() << "Factory::doCreate(const tnt::Compident& ci,\n"
+                "tnt::Component* _component_" << maincomp.getName() << "Factory::doCreate(const tnt::Compident& ci,\n"
                 "  const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
                 "{\n"
-                "  return new " << maincomp.getName() << "(ci, um, cl);\n"
+                "  return new _component_" << maincomp.getName() << "(ci, um, cl);\n"
                 "}\n\n"
-                "void " << maincomp.getName() << "Factory::doConfigure(const tnt::Tntconfig& config)\n"
+                "void _component_" << maincomp.getName() << "Factory::doConfigure(const tnt::Tntconfig& config)\n"
                 "{\n"
                 "  // <%config>\n";
         for (variable_declarations::const_iterator it = configs.begin();
              it != configs.end(); ++it)
-          it->getConfigInit(code, maincomp.getName());
+          it->getConfigInit(code, "_component_" + maincomp.getName());
         code << "  // </%config>\n"
                 "}\n\n"
-                "static " << maincomp.getName() << "Factory factory;\n\n";
+                "static _component_" << maincomp.getName() << "Factory factory;\n\n";
       }
     }
 
     void Generator::getCppBody(std::ostream& code) const
     {
-      std::string classname = maincomp.getName();
+      std::string classname = "_component_" + maincomp.getName();
 
       getFactoryDeclaration(code);
 
@@ -510,7 +511,7 @@ namespace tnt
         code << "  rawData.release();\n";
 
       code << "}\n\n"
-              "unsigned " << classname << "::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, cxxtools::QueryParams& qparam)\n"
+              "unsigned " << classname << "::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)\n"
            << "{\n";
 
       if (isDebug())
@@ -546,7 +547,7 @@ namespace tnt
                   "  reply.setDirectMode();\n";
 
         code << '\n';
-        maincomp.getBody(code);
+        maincomp.getBody(code, linenumbersEnabled);
         code << "}\n\n";
       }
       else
@@ -578,7 +579,7 @@ namespace tnt
       }
 
       if (haveCloseComp)
-        closeComp.getDefinition(code, externData);
+        closeComp.getDefinition(code, externData, linenumbersEnabled);
 
       if (!attr.empty())
       {
@@ -595,7 +596,7 @@ namespace tnt
 
       for (subcomps_type::const_iterator i = subcomps.begin();
            i != subcomps.end(); ++i)
-        i->getDefinition(code, isDebug(), externData);
+        i->getDefinition(code, isDebug(), externData, linenumbersEnabled);
     }
 
     void Generator::printLine(std::ostream& out) const
@@ -606,11 +607,11 @@ namespace tnt
 
     void Generator::getHeader(std::ostream& header, const std::string& filename) const
     {
-      std::string classname = maincomp.getName();
+      std::string classname = "_component_" + maincomp.getName();
 
       getIntro(header, filename);
-      header << "#ifndef ECPP_COMPONENT_" << classname << "_H\n"
-                "#define ECPP_COMPONENT_" << classname << "_H\n\n";
+      header << "#ifndef ECPP_COMPONENT_" << maincomp.getName() << "_H\n"
+                "#define ECPP_COMPONENT_" << maincomp.getName() << "_H\n\n";
 
       getHeaderIncludes(header);
       header << '\n';
